@@ -3,18 +3,19 @@ package MOP4Import::Declare;
 use 5.010;
 use strict;
 use warnings qw(FATAL all NONFATAL misc);
-our $VERSION = '0.051_001';
+our $VERSION = '0.070_001'; BEGIN {$VERSION = '0.062'};
 use Carp;
 use mro qw/c3/;
 
 use constant DEBUG => $ENV{DEBUG_MOP4IMPORT};
+BEGIN {
+  print STDERR "\nUsing ".__PACKAGE__. " = $VERSION (file '"
+    . __FILE__ . "')\n"
+    if DEBUG;
+}
 
 use Sub::Util ();
 
-print STDERR "\nUsing ".__PACKAGE__. " = $VERSION (file '"
-  . __FILE__ . "')\n"
-  if DEBUG;
-
 use MOP4Import::Opts
   qw/
       Opts
@@ -23,7 +24,12 @@ use MOP4Import::Opts
     /;
 use MOP4Import::Util;
 use MOP4Import::FieldSpec;
-use MOP4Import::NamedCodeAttributes qw(MODIFY_CODE_ATTRIBUTES /^m4i_CODE_ATTR_/);
+use MOP4Import::NamedCodeAttributes
+  qw(MODIFY_CODE_ATTRIBUTES
+     FETCH_CODE_ATTRIBUTES
+     /^m4i_CODE_ATTR_/
+     declare_code_attributes
+  );
 
 #========================================
 
@@ -36,32 +42,68 @@ sub import :MetaOnly {
 
   my Opts $opts = m4i_opts([caller]);
 
-  @decls = $myPack->default_exports unless @decls;
+  @decls = $myPack->default_exports($opts) unless @decls;
 
-  $myPack->dispatch_declare($opts, $myPack->always_exports, @decls);
+  $myPack->dispatch_declare($opts, $myPack->always_exports($opts), @decls);
+
+  my $tasks;
+  if ($tasks = $opts->{delayed_tasks} and @$tasks) {
+    print STDERR " Calling delayed tasks for $opts->{destpkg}\n" if DEBUG;
+    $_->($opts) for @$tasks;
+  }
 
   m4i_log_end($opts->{callpack}) if DEBUG;
 }
 
-sub file_line_of {
+sub m4i_file_line_of :method :MetaOnly {
   (my $myPack, my Opts $opts) = @_;
   " at $opts->{filename} line $opts->{line}";
 }
 
+sub m4i_stash :method :MetaOnly {
+  (my $myPack, my Opts $opts) = @_;
+  $opts->{stash}{$myPack} //= +{};
+}
+
 #
-# This serves as @EXPORT
+# `default_exports` is a hook to list exported symbols
+# when the module is used without arguments, like "use Foo".
+# Default implementation get them from `@EXPORT`.
 #
 sub default_exports {
-  ();
+  (my $myPack, my Opts $opts) = @_;
+  my $symtab = MOP4Import::Util::symtab($myPack);
+  my $sym = $symtab->{EXPORT};
+  if ($sym and my $export = *{$sym}{ARRAY}) {
+    print STDERR "HAS \@EXPORT: $myPack" if DEBUG;
+    @$export
+  } else {
+    print STDERR "NO \@EXPORT: $myPack"  if DEBUG;
+    ();
+  }
+}
+
+sub declare_default_exports {
+  (my $myPack, my Opts $opts, my (@decls)) = m4i_args(@_);
+
+  my $export = MOP4Import::Util::ensure_symbol_has_array(
+    globref($opts->{objpkg}, 'EXPORT')
+  );
+
+  push @$export, @decls;
 }
 
 sub always_exports {
+  (my $myPack, my Opts $opts) = @_;
   qw(-strict);
 }
 
 sub dispatch_declare :MetaOnly {
   (my $myPack, my Opts $opts, my (@decls)) = m4i_args(@_);
 
+  print STDERR "$myPack->dispatch_declare("
+    .terse_dump($opts, @decls).")\n" if DEBUG;
+
   foreach my $declSpec (@decls) {
 
     croak "Undefined pragma!" unless defined $declSpec;
@@ -183,7 +225,7 @@ sub dispatch_declare_pragma :MetaOnly {
     $sub->($myPack, $opts, @args);
   } else {
     croak "No such pragma: \`use $myPack\ [".terse_dump($pragma)."]`"
-      . $myPack->file_line_of($opts);
+      . $myPack->m4i_file_line_of($opts);
   }
 }
 
@@ -240,7 +282,7 @@ sub declare_parent :MetaOnly {
   (my $myPack, my Opts $opts, my (@base)) = m4i_args(@_);
 
   foreach my $fn (@base) {
-    (my $cp = $fn) =~ s{::|'}{/}g;
+    (my $cp = $fn) =~ s{::|\'}{/}g;
     require "$cp.pm";
   }
 
@@ -372,10 +414,14 @@ sub declare_fields :MetaOnly {
   my $extended = fields_hash($opts->{objpkg});
   my $fields_array = fields_array($opts->{objpkg});
 
+  my $isFirst = not $myPack->JSON_TYPE_HANDLER->lookup_json_type($opts->{objpkg});
+
   # Import all fields from super class
   foreach my $super_class (@{*{globref($opts->{objpkg}, 'ISA')}{ARRAY}}) {
     my $super = fields_hash($super_class);
     next unless $super;
+
+
     my $super_names = fields_array($super_class);
     my @names = @$super_names ? @$super_names : keys %$super;
     foreach my $name (@names) {
@@ -388,15 +434,10 @@ sub declare_fields :MetaOnly {
         ref $origin ? $origin->clone : $origin;
       };
       push @$fields_array, $name;
-      if (ref $anyFieldSpec) {
-        my FieldSpec $fs = $anyFieldSpec;
-        if ($fs->{json_type}) {
-          $myPack->declare___field_with_json_type(
-            $opts, $fs, json_type => $fs->{json_type}
-          );
-        }
-      }
     }
+
+    $myPack->JSON_TYPE_HANDLER->inherit_json_type($opts->{objpkg}, $super_class) if $isFirst;
+
   }
 
   {
@@ -421,6 +462,8 @@ sub declare_fields :MetaOnly {
 
   print STDERR "  FieldSpec is: $field_class\n" if DEBUG;
 
+  $myPack->JSON_TYPE_HANDLER->declare_json_type_record($opts->{objpkg});
+
   $myPack->declare___field($opts, $field_class, ref $_ ? @$_ : $_) for @fields;
 
   $opts->{objpkg}; # XXX:
@@ -451,15 +494,15 @@ sub declare___field :MetaOnly {
       . ".$k";
   }
 
-  my FieldSpec $obj = $extended->{$name}
+  my FieldSpec $fs = $extended->{$name}
     = $field_class->new(@early, name => $name);
-  $obj->{package} = $opts->{objpkg};
-  print STDERR " with $myPack $field_class => ", terse_dump($obj), "\n"
+  $fs->{package} = $opts->{objpkg};
+  print STDERR " with $myPack $field_class => ", terse_dump($fs), "\n"
     if DEBUG;
   push @$fields_array, $name;
 
   # Create accessor for all public fields.
-  if ($name =~ /^[a-z]/i and not $obj->{no_getter}) {
+  if ($name =~ /^[a-z]/i and not $fs->{no_getter}) {
     if (my $sym = MOP4Import::Util::symtab($opts->{objpkg})->{$name}) {
       if (*{$sym}{CODE}) {
         croak "Accessor $opts->{objpkg}::$name is redefined!\nIf you really want to define the accessor by hand, please specify fields spec like: [$name => no_getter => 1, ...].";
@@ -473,10 +516,17 @@ sub declare___field :MetaOnly {
 
   foreach my $delayed (@delayed) {
     my ($sub, $k, $v) = @$delayed;
-    $sub->($myPack, $opts, $obj, $k, $v);
+    $sub->($myPack, $opts, $fs, $k, $v);
   }
 
-  $obj;
+  if ($fs->{name} !~ /^_/) {
+    $myPack->JSON_TYPE_HANDLER->register_json_type_of_field(
+      $opts, $opts->{objpkg}, $fs->{name},
+      $fs->{json_type} || $opts->{default_json_type} || 'string'
+    );
+  }
+
+  $fs;
 }
 
 sub declare___field_with_default :MetaOnly {
@@ -496,18 +546,6 @@ sub JSON_TYPE_HANDLER {
   'MOP4Import::Util::JSON_TYPE';
 }
 
-sub declare___field_with_json_type :MetaOnly {
-  (my $myPack, my Opts $opts, my FieldSpec $fs, my ($kw, $typeSpec)) = m4i_args(@_);
-
-  print STDERR "  $fs->{name} json_type: spec=", terse_dump($typeSpec), "\n" if DEBUG;
-
-  my $json_type = $myPack->JSON_TYPE_HANDLER
-    ->register_json_type_of_field($opts->{objpkg}, $fs->{name}, $typeSpec);
-  $fs->{json_type} = $json_type;
-
-  #" result=", terse_dump($json_type), "\n"
-}
-
 sub declare_alias :MetaOnly {
   (my $myPack, my Opts $opts, my ($name, $alias)) = m4i_args(@_);
   print STDERR " Declaring alias $name in $opts->{destpkg} as $alias\n" if DEBUG;
@@ -537,6 +575,8 @@ sub declare_defaults :MetaOnly {
     $myPack->declare_private_constant($opts, $fn, $v);
   }
 }
+*declare_override_defaults = *declare_defaults;
+*declare_override_defaults = *declare_defaults;
 
 sub declare_map_methods :MetaOnly {
   (my $myPack, my Opts $opts, my (@pairs)) = m4i_args(@_);
@@ -568,7 +608,7 @@ sub declare_carp_not :MetaOnly {
   #
   # Below does equiv of `our @CARP_NOT = qw/ MOP4Import::Util /;`
   #
-  __PACKAGE__->declare_carp_not(MOP4Import::Opts::m4i_fake(__PACKAGE__),
+  __PACKAGE__->declare_carp_not(MOP4Import::Opts::m4i_fake_opts(__PACKAGE__),
                                 qw/
                                    MOP4Import::Util
                                    /
