@@ -5,6 +5,7 @@ use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 
 use MOP4Import::Pairs -as_base, qw/Opts m4i_opts/;
+# XXX: [carp_not => MOP4Import::Util]
 use MOP4Import::Declare::Type -as_base;
 use MOP4Import::Util;
 
@@ -19,6 +20,12 @@ sub import {
 
   $myPack->dispatch_pairs_as_declare(type => $opts, @_);
 
+  my $tasks;
+  if ($tasks = $opts->{delayed_tasks} and @$tasks) {
+    print STDERR " Calling delayed tasks for $opts->{destpkg}\n" if DEBUG;
+    $_->($opts) for @$tasks;
+  }
+
   m4i_log_end($opts->{callpack}) if DEBUG;
 }
 
